@@ -1,19 +1,20 @@
 package List::Pairwise;
+use 5.006;
 use strict;
-#use warnings;
-use vars qw($VERSION %EXPORT_TAGS @EXPORT_OK);
+use warnings;
 use Exporter;
 
-$VERSION= '0.23';
+our $VERSION= '0.25';
 
-%EXPORT_TAGS = ( 
+our %EXPORT_TAGS = ( 
 	all => [ qw(
 		mapp grepp firstp lastp
 		map_pairwise grep_pairwise first_pairwise last_pairwise
+		pair
 	) ],
 );
 
-@EXPORT_OK = ( @{ $EXPORT_TAGS{all} } );
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{all} } );
 
 sub import {
 	my $pkg = caller();
@@ -26,13 +27,20 @@ sub import {
 	goto &Exporter::import
 }
 
-sub mapp (&@) {
-	my $code = shift;
-	
-	if (@_&1) {
+sub _carp_odd {
+	if (warnings::enabled('misc')) {
 		require Carp;
-		Carp::croak("Odd number of elements in list")
+		Carp::carp('Odd number of elements in &', [caller(1)]->[3], ' arguments');
 	}
+}
+
+sub mapp (&@) {
+	unless (@_&1) {
+		_carp_odd;
+		push @_, undef;
+		goto \&mapp;
+	}
+	my $code = shift;
 
 	# Localise $a and $b
 	# (borrowed from List-MoreUtils)
@@ -43,8 +51,7 @@ sub mapp (&@) {
 	};
 	local(*$caller_a, *$caller_b);
 
-	#no warnings;
-	local $^W = 0;
+	no warnings;
 	
 	if (wantarray) {
 		# list context
@@ -65,12 +72,12 @@ sub mapp (&@) {
 }
 
 sub grepp (&@) {
-	my $code = shift;
-	
-	if (@_&1) {
-		require Carp;
-		Carp::croak("Odd number of elements in list")
+	unless (@_&1) {
+		_carp_odd;
+		push @_, undef;
+		goto \&grepp;
 	}
+	my $code = shift;
 
 	# Localise $a and $b
 	# (borrowed from List-MoreUtils)
@@ -81,8 +88,7 @@ sub grepp (&@) {
 	};
 	local(*$caller_a, *$caller_b);
 
-	#no warnings;
-	local $^W = 0;
+	no warnings;
 
 	if (wantarray) {
 		# list context
@@ -107,12 +113,12 @@ sub grepp (&@) {
 }
 
 sub firstp (&@) {
-	my $code = shift;
-	
-	if (@_&1) {
-		require Carp;
-		Carp::croak("Odd number of elements in list")
+	unless (@_&1) {
+		_carp_odd;
+		push @_, undef;
+		goto \&firstp;
 	}
+	my $code = shift;
 
 	# Localise $a and $b
 	# (borrowed from List-MoreUtils)
@@ -123,8 +129,7 @@ sub firstp (&@) {
 	};
 	local(*$caller_a, *$caller_b);
 
-	#no warnings;
-	local $^W = 0;
+	no warnings;
 	
 	if (wantarray) {
 		# list context
@@ -139,12 +144,12 @@ sub firstp (&@) {
 }
 
 sub lastp (&@) {
-	my $code = shift;
-	
-	if (@_&1) {
-		require Carp;
-		Carp::croak("Odd number of elements in list")
+	unless (@_&1) {
+		_carp_odd;
+		push @_, undef;
+		goto \&lastp;
 	}
+	my $code = shift;
 
 	# Localise $a and $b
 	# (borrowed from List-MoreUtils)
@@ -155,8 +160,7 @@ sub lastp (&@) {
 	};
 	local(*$caller_a, *$caller_b);
 
-	#no warnings;
-	local $^W = 0;
+	no warnings;
 	
 	if (wantarray) {
 		# list context
@@ -170,6 +174,14 @@ sub lastp (&@) {
 	}
 }
 
+sub pair {
+	_carp_odd if @_&1;
+	return @_
+		? map [ @_[$_*2, $_*2 + 1] ] => 0 .. ($#_>>1)
+		: wantarray ? () : 0
+	;
+}
+
 #sub truep   (&@) { scalar &grepp(@_)      }
 #sub falsep  (&@) { (@_-1)/2 - &grepp(@_)  }
 #sub allp    (&@) { (@_-1)/2 == &grepp(@_) }
